#################################
# Sim01_ZINB_Final.r
# Fit ZINB fixed effects model with 3 Covs for Simulation Study 1
# Sept 15, 2018
#################################
library(BayesLogit)   # For rpg function -- install from ZIP file
library(mvtnorm)	
library(msm)          # For tnorm function

#################
# Generate Data #
#################
  set.seed(041317)
  n<-1000
  x1<-rnorm(n)
  x2<-rbinom(n,1,.5)
  x3<-sample(0:2,n,replace=T)
  X<-cbind(rep(1,n),x1,x2,x3)
  p<-ncol(X)

# Binomial part
  alpha<-c(0.5,0.5,-0.25,.25)
  eta1<-X%*%alpha
  phi<-exp(eta1)/(1+exp(eta1)) # 1 - Prob of "Structural Zero"
  u<-rbinom(n,1,phi)           # at-risk indicator
  n1<-sum(u)           
  pstruct0<-(n-n1)/n           # Proportion of structural zeros

# Count Part 
  beta<-c(0.5,-1,.75,-.25)
  eta2<-X[u==1,]%*%beta
  r<-1                      
  pi<-exp(eta2)/(1+exp(eta2))  # Prob of success
  mu<-r*pi/(1-pi)
  y<-rep(0,n)
  y[u==1]<-rnbinom(n1,r,mu=mu)
  pzero<-length(y[y==0])/n     # Proportion of zeros

# Histogram 
  tmp<-table(y)/n*100 # convert to percents (divide by 1000 multiply by 10)
  barplot(tmp, ylab="Percent",xlab="Count",col="lightblue")

# Priors -- note diffuse uniform for r
  beta0<-alpha0<-rep(0,p)
  T0a<-diag(0.01,p)          # Prior precision for alpha -- tightening can help in bordeline cases
  T0b<-diag(0.01,p)          # Prior precision for beta

# Inits 
  beta<-alpha<-rep(0,p)
  r<-1
  Acc<-0  
  y1<-rbinom(n,1,.5)        # At risk indicator (W in paper)
  y1[y>0]<-1                # If y>0, then patient is at risk w.p. 1
  n0<-length(y[y==0])       # Number of observed 0's
  q<-rep(.5,n)              # 1-p=1/(1+exp(X%*%alpha)), used for updating y1
  s<-0.025      

############
# Num Sims #
############
  nsim<-1500			          # Number of MCMC Iterations (Used 50500 in paper)
  thin<-1			              # Thinning interval
  burn<-500    		          # Burnin
  lastit<-(nsim-burn)/thin	# Last stored value

#########
# Store #
#########
  Beta<-Alpha<-matrix(0,lastit,p)
  R<-rep(0,lastit) 
  R2<-rep(0,lastit)         # To store Gibbs updates
  
########
# MCMC #
########
tmp<-proc.time()

for (i in 1:nsim){
  
# Update alpha
  mu<-X%*%alpha
  w<-rpg(n,1,mu)
  z<-y1-1/2                               # Or z=(y-1/2)/w with m=v%*%(T0%*%beta0+t(sqrt(w)*X)%*%sqrt(w)*z)
  v<-solve(crossprod(X*sqrt(w))+T0a)      # Or solve(X%*%W%*%X), which is slower
  m<-v%*%(T0a%*%alpha0+t(X)%*%z)
  alpha<-c(rmvnorm(1,m,v))

# Update latent class indicator y1 (w in paper)
  eta<-X%*%alpha
  phi<-exp(eta)/(1+exp(eta))              # at-risk probability
  theta<-phi*(q^r)/(phi*(q^r)+1-phi)      # Conditional prob that y1=1 given y=0 -- i.e. Pr(chance zero|observed zero)
  y1[y==0]<-rbinom(n0,1,theta[y==0])      # If y=0, then draw a "chance zero" w.p. theta, if y=1, then y1=1
  n1<-sum(y1)
  
# Update rho                              # Proposal variance
  rnew<-rtnorm(1,r,sqrt(s),lower=0)       # Treat r as continuous
  ratio<-sum(dnbinom(y[y1==1],rnew,q[y1==1],log=T))-sum(dnbinom(y[y1==1],r,q[y1==1],log=T))+   
              dtnorm(r,rnew,sqrt(s),0,log=T) - dtnorm(rnew,r,sqrt(s),0,log=T) # Note: Uniform prior for r
                                                                              #       Proposal not symmetric 
  if (log(runif(1))<ratio) {
    r<-rnew
    Acc<-Acc+1
  }
  
# Update beta
  eta1<-X[y1==1,]%*%beta
  w<-rpg(n1,y[y1==1]+r,eta1)                              # Polya weights
  z<-(y[y1==1]-r)/(2*w)                                   # Latent "response"
  v<-solve(crossprod(X[y1==1,]*sqrt(w))+T0b)               
  m<-v%*%(T0b%*%beta0+t(sqrt(w)*X[y1==1,])%*%(sqrt(w)*z))  
  beta<-c(rmvnorm(1,m,v))
  eta<-X%*%beta
  q<-1/(1+exp(eta))

# Update r2 using Gibbs as in Dadaneh et al and Zhou and Carin #

# Update latent counts, l        
  l<-rep(0,n1)
  ytmp<-y[y1==1]
  for(j in 1:n1) l[j]<-sum(rbinom(ytmp[j],1,round(r/(r+1:ytmp[j]-1),6))) # Could try to avoid loop; rounding avoids numerical stability
  
# Update r from conjugate gamma distribution given l and psi
  eta<-X[y1==1,]%*%beta
  psi<-exp(eta)/(1+exp(eta))
  r2<-rgamma(1,0.01+sum(l),0.01-sum(log(1-psi))) # Gamma(0.01,0.01) prior for r
  
# Store
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Alpha[j,]<-alpha
    Beta[j,]<-beta
    R[j]<-r	
    R2[j]<-r2
  }
  
  if (i%%100==0) print(i)

}

tot.time<-proc.time()-tmp  # 9.8 minutes to run 50K

###########
# Results #
###########
  malpha<-colMeans(Alpha)
  salpha<-apply(Alpha,2,sd)
  qalpha<-apply(Alpha,2,quantile,prob=c(0.025,0.975))
  mbeta<-colMeans(Beta)
  sbeta<-apply(Beta,2,sd)
  qbeta<-apply(Beta,2,quantile,prob=c(0.025,0.975))
  mr<-mean(R)
  qr<-quantile(R,prob=c(0.025,0.975))
  mr2<-mean(R2)
  qr2<-quantile(R2,prob=c(0.025,0.975))

  malpha
  mbeta
  mr
  mr2

###############
# Trace plots #
###############
  par(mfrow=c(3,2))
  plot(1:lastit,Alpha[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[10]))
  abline(h=malpha[1],col="blue4")
  
  plot(1:lastit,Alpha[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[11]))
  abline(h=malpha[2],col="blue4")
  
  plot(1:lastit,Beta[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[20]))
  abline(h=mbeta[1],col="blue4")
  
  plot(1:lastit,Beta[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[22]))
  abline(h=mbeta[2],col="blue4")
  
  plot(1:lastit,Beta[,3],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[23]))
  abline(h=mbeta[3],col="blue4")
  
  plot(1:lastit,R,type="l",col="lightgreen",xlab="Stored Iteration",ylab=expression(r))
  abline(h=mr,col="blue4")
  