#################################
# Sim02_Final.r
# Longitudinal Model
# Add binary time-invariant covariate
# Final Simulation 
# Sept 16, 2018                
#################################
library(BayesLogit)   # For rpg function -- install from ZIP file
library(mvtnorm)	
library(MCMCpack)     # For Iwish update of Sigmab
library(msm)          # For tnorm function

#################
# Generate Data #
#################
  set.seed(100717)
  n<-1000					              # sample size
  nis<-sample(1:10,n,replace=T) # number obs per subject 
  id<-rep(1:n,nis)
  N<-length(id)
  t<-rep(0,N)
  for (i in 1:n) t[id==i]<-rnorm(nis[i],2)#sort(sample(1:15,nis[i]))
  x<-rbinom(n,1,.5)
  X<-cbind(1,rep(x,nis),t)
  p<-ncol(X)

# Random effects
  Sigma<-Sigmab<-matrix(c(0.5,.25,.25,0.75),2,2)
  true.rho<-Sigma[1,2]/sqrt(Sigma[1,1]*Sigma[2,2])
  b<-rmvnorm(n,sigma=Sigma)
  b1<-b[,1]
  b2<-b[,2]

# Binomial part
  alpha<-c(0.25,-0.25,0.25)
  eta1<-X%*%alpha+rep(b1,nis)
  phi<-exp(eta1)/(1+exp(eta1)) # 1 - Prob of "Structural Zero"
  u<-rbinom(N,1,phi)           # at-risk indicator
  N1<-sum(u)          
  pstruct0<-1-mean(u)          # Proportion of structural zeros
  
# Count Part 
  beta<-c(0.5,-.25,0.25)
  tmp<-u
  tmp[u==0]<-0
  nis1<-tapply(tmp,id,sum)
  eta2<-X[u==1,]%*%beta+rep(b2,nis1)
  r<-1.25              
  pi<-exp(eta2)/(1+exp(eta2))      # Prob of success
  mu<-r*pi/(1-pi)           
  y<-rep(0,N)
  y[u==1]<-rnbinom(N1,r,mu=mu)
  pzero<-length(y[y==0])/N     # Proportion of zeros
  qy<-quantile(y[u==1])

  tmp<-table(y)/N*100  # convert to %s (divide by 1000 multiply by 10)
  barplot(tmp, ylab="Percent",xlab="Count",col="lightblue")

 # Priors -- uniform or gamma(0.01,0.01) for r depending on whether MH or Gibbs
   alpha0<-beta0<-rep(0,p)
   T0a<-diag(.01,p)
   T0b<-diag(.01,p)

 # Inits 
   beta<-alpha<-rep(0,p)
   b1<-b2<-rnorm(n)			    # Random effects
   rho<-0                   # Random effect correlation
   sigmab1<-sigmab2<-1			# Random Effects SDs
   r<-1
   Acc<-0  
   y1<-rep(0,N)             # At risk indicator (W in paper)
   y1[y>0]<-1               # If y>0, then at risk w.p. 1
   N0<-length(y[y==0])      # Number of observed 0's
   q<-rep(.5,N)             # 1-p=1/(1+exp(X%*%alpha)), used for updating y1
   s<-0.003                 # Proposal variance

############
# Num Sims #
############
  nsim<-1500			          # Number of MCMC Iterations (Used 50500 in paper)
  thin<-1			              # Thinning interval
  burn<-500		              # Burnin
  lastit<-(nsim-burn)/thin	# Last stored value

#########
# Store #
#########
  Beta<-Alpha<-matrix(0,lastit,p)
  R<-Sigmab1<-Sigmab2<-rep(0,lastit)
  R2<-rep(0,lastit)              # For Gibbs update of R
  Sigb<-matrix(0,lastit,4)
  
########
# MCMC #
########
tmp<-proc.time()

for (i in 1:nsim){

# Update alpha  
  mu<-X%*%alpha+rep(b1,nis)
  w<-rpg(N,1,mu)
  z<-(y1-1/2)/w           
  v<-solve(crossprod(sqrt(w)*X)+T0a)  
  m<-v%*%(T0a%*%alpha0+t(sqrt(w)*X)%*%(sqrt(w)*(z-rep(b1,nis))))
  alpha<-c(rmvnorm(1,m,v))

# Update b1
  priorprec<-1/(sigmab1^2*(1-rho^2))  # Prior precision of b1|b2
  priormean<-rho*sigmab1/sigmab2*b2   # Prior mean of b1|b2 
  vb<-1/(priorprec+tapply(w,id,sum))			
  mb<-vb*(priorprec*priormean+tapply(w*(z-X%*%alpha),id,sum)) 
  b1<-rnorm(n,mb,sqrt(vb)) 	
  
# Update at risk indicator y1 (W in paper)
  eta1<-X%*%alpha+rep(b1,nis)
  eta2<-X%*%beta+rep(b2,nis)
  phi<-pmax(0.01,pmin(0.99,exp(eta1)/(1+exp(eta1))))  # at-risk probability
  q<-1/(1+exp(eta2))                      # Pr(y=0|y1=1)
  theta<-phi*(q^r)/(phi*(q^r)+1-phi)      # Conditional prob that y1=1 given y=0 -- i.e. Pr(chance zero|observed zero)
  y1[y==0]<-rbinom(N0,1,theta[y==0])      # If y=0, then draw a "chance zero" w.p. theta, otherwise y1=1
  N1<-sum(y1)
  nis1<-tapply(y1,id,sum)
  
# Update r                          # Proposal variance
  rnew<-rtnorm(1,r,sqrt(s),lower=0)       # Treat r as continuous
  ratio<-sum(dnbinom(y[y1==1],rnew,q[y1==1],log=T))-sum(dnbinom(y[y1==1],r,q[y1==1],log=T))+   
    dtnorm(r,rnew,sqrt(s),0,log=T) - dtnorm(rnew,r,sqrt(s),0,log=T)   # Note: Prior for r uniform 
                                                                      #       Proposal not symmetric 
  if (log(runif(1))<ratio) {
    r<-rnew
    Acc<-Acc+1
  }
  
# Update r2 using Gibbs as in Dadaneh et al and Zhou and Carin
# Update latent counts, l        
  l<-rep(0,N1)
  ytmp<-y[y1==1]
  for(j in 1:N1) l[j]<-sum(rbinom(ytmp[j],1,round(r/(r+1:ytmp[j]-1),6))) # Could try to avoid loop; rounding avoids numerical stability
  
# Update r from conjugate gamma distribution given l and psi
  eta<-X[y1==1,]%*%beta+rep(b2,nis1)
  psi<-exp(eta)/(1+exp(eta))
  r2<-rgamma(1,0.01+sum(l),0.01-sum(log(1-psi)))
  
# Update beta
  eta<-X[y1==1,]%*%beta+rep(b2,nis1)
  w<-rpg(N1,y[y1==1]+r,eta)                               # Polya weights
  z<-(y[y1==1]-r)/(2*w)                                      
  v<-solve(crossprod(X[y1==1,]*sqrt(w))+T0b)               
  m<-v%*%(T0b%*%beta0+t(sqrt(w)*X[y1==1,])%*%(sqrt(w)*(z-rep(b2,nis1))))  
  beta<-c(rmvnorm(1,m,v))

# Update b2
  n1<-length(nis1[nis1>0])
  
  priorprec<-1/(sigmab2^2*(1-rho^2))            # Prior precision of b2|b1
  priormean<-rho*sigmab2/sigmab1*b1             # Prior mean of b2|b1 

# Sample from conditional prior if y=0
  b2[nis1==0]<-rnorm(n-n1,priormean[nis1==0],sqrt(1/priorprec))
  
  vb<-1/(priorprec+tapply(w,id[y1==1],sum))	
  mb<-vb*(priorprec*priormean[nis1>0]+tapply(w*(z-X[y1==1,]%*%beta),id[y1==1],sum)) 
  b2[nis1>0]<-rnorm(n1,mb,sqrt(vb)) 
  
# Update Sigmab
  b<-cbind(b1,b2)
  Sigmab<-riwish(2+n,diag(2)+crossprod(b))
  sigmab1<-sqrt(Sigmab[1,1])          # Sd b1
  sigmab2<-sqrt(Sigmab[2,2])          # Sd b2
  rho<-Sigmab[1,2]/(sigmab1*sigmab2)  # Corr(b1,b2)
  
# Store
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Alpha[j,]<-alpha
    Beta[j,]<-beta
    R[j]<-r		
    R2[j]<-r2	
    Sigb[j,]<-c(Sigmab)
  }
  
  if (i%%100==0)  print(i)

  
}

tot.time<-proc.time()-tmp  # 29 minutes to run 50K without R2, takes 78 seconds to run 2500 sec vs 54 for SAS

###########
# Results #
###########
  malpha<-colMeans(Alpha)
  salpha<-apply(Alpha,2,sd)
  qalpha<-apply(Alpha,2,quantile,prob=c(0.025,0.975))
  mbeta<-colMeans(Beta)
  sbeta<-apply(Beta,2,sd)
  qbeta<-apply(Beta,2,quantile,prob=c(0.025,0.975))
  mr<-mean(R)
  qr<-quantile(R,prob=c(0.025,0.975))
  mr2<-mean(R2)
  qr2<-quantile(R2,prob=c(0.025,0.975))
  msigb<-colMeans(Sigb)
  qsigb<-apply(Sigb,2,quantile,prob=c(0.025,0.975))
  mrho<-mean(Sigb[,2]/sqrt(Sigb[,1]*Sigb[,4]))
  qrho<-quantile(Sigb[,2]/sqrt(Sigb[,1]*Sigb[,4]),prob=c(0.025,0.975))

  malpha
  mbeta
  mr
  mr2
  msigb
  mrho

###############
# Trace plots #
###############
  par(mfrow=c(3,2))
  plot(1:lastit,Alpha[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[10]))
  abline(h=malpha[1],col="blue4")
  
  plot(1:lastit,Beta[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[20]))
  abline(h=mbeta[1],col="blue4")

  plot(1:lastit,R,type="l",col="lightgreen",xlab="Iteration",ylab=expression(r))
  abline(h=mr,col="blue4")

  plot(1:lastit,Sigb[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[11]))
  abline(h=mean(Sigb[,1]),col="blue4")

  plot(1:lastit,Sigb[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[12]))
  abline(h=mean(Sigb[,2]),col="blue4")

  plot(1:lastit,Sigb[,4],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[22]))
  abline(h=mean(Sigb[,4]),col="blue4")
