#################################
# Sim03_Final.r
# Fit ZINB with Random intrecepts and slopes for both parts
# Adds mcse, DIC and Gibbs Update for r
# Infile: "I:/Brian/Negative Binomial/R/A.txt" -- Visn7 Adj Matrix 
# Sep 18, 2018                
#################################
library(BayesLogit)   # For rpg function -- install from ZIP file
library(mvtnorm)	
library(MCMCpack)     # For Iwish update of Sigmab
library(msm)          # For tnorm function
library(spam)         # For sparse Matrix

##################
# Adjacency Info #
##################
  A<-dget("I:/Brian/ZINB/Simulations/A.txt") 		# Visn7 Adj Matrix 
  m<-apply(A,1,sum)	  # No. neighbors for each county
  
##################
# Generate Data  #
##################
  set.seed(091017)
  n<-nrow(A)			    # Number of spatial units
  nis<-rep(250,n) 		# Number of individuals per county; here it's balanced -- 50 per county per year
                      # Note: may need to lower proposal variance, s, below as n_i increases 
  id<-rep(1:n,nis)  
  N<-length(id) 		  # Total number of observations (50 subjects per county)

########################
#  Spatial Effects     #
########################
  kappa<-.999999			  	            # Spatial dependence parameter ~ 1 for intrinsic CAR
  cov<-matrix(c(.5,.10,.10,-.10,
                .10,.15,.10,.10,
                .10,.10,.5,.10,
                -.10,.10,.10,.15),4,4)# Conditional Cov of phi1 and phi2 given phi_{-i} 
  Q<-as.spam(diag(m))-kappa*as.spam(A)			     
  covphi<-solve(Q)%x%cov			        # 3n x 3n covariance of phis
  phi<-rmvnorm(1,sigma=covphi)		    # 3n vector of spatial effects
  phitmp<-matrix(phi,ncol=4,byrow=T)  # n x 3 matrix of spatial effects

  true.phi1<-phi1<-phitmp[,1]-mean(phitmp[,1])   # n x 1 phi1 vector -- Centered
  true.phi2<-phi2<-phitmp[,2]-mean(phitmp[,2])   # n x 1 phi2 vector, etc.
  true.phi3<-phi3<-phitmp[,3]-mean(phitmp[,3])
  true.phi4<-phi4<-phitmp[,4]-mean(phitmp[,4])
  Phi1<-rep(phi1,nis)
  Phi2<-rep(phi2,nis)
  Phi3<-rep(phi3,nis)
  Phi4<-rep(phi4,nis)
  true.phi<-cbind(true.phi1,true.phi2,true.phi3,true.phi4)
  
#################
# Fixed Effects #
#################
   x<-sample(seq(0,4),N,replace=T)     # x = year (0,1,2,3,4) -- if you relabel, need to relabel below
                                       # This is okay but not exactly balanced by year. Should be x[1:13600]<-0, x[13601:27200]<-2, etc Need to update
   X<-cbind(1,x)                       # Design matrix, can add additional covariates (e.g., race, age, gender)
   p<-ncol(X)
  
 # Binomial part
   alpha<-c(-0.25,0.25)
   eta1<-X%*%alpha+Phi1+Phi2*x
   pi<-exp(eta1)/(1+exp(eta1))        # 1-pr("structural zero")
   u<-rbinom(N,1,pi)                  # at-risk indicator
   N1<-sum(u)          
   pstruct0<-1-mean(u)                # Proportion of structural zeros
  
 # Count Part 
   beta<-c(.5,-.25)
   tmp<-u
   tmp[u==0]<-0                       # If in structural group then set tmp to 0
   nis1<-tapply(tmp,id,sum)           # Number of at risk observations in each county
   
   eta2<-X[u==1,]%*%beta+Phi3[u==1]+Phi4[u==1]*x[u==1] # Linear predictor for count part
   r<-1                               # NB dispersion 
   psi<-exp(eta2)/(1+exp(eta2))       # Prob of success
   mu<-r*psi/(1-psi)                  # NB mean
   
   y<-rep(0,N)                        # Response
   y[u==1]<-rnbinom(N1,r,mu=mu)       # If at risk, draw from NB
   pzero<-length(y[y==0])/N           # Proportion of zeros
  # pzero
  # pstruct0/pzero
  # table(y)
  # mean(mu)
  # quantile(tapply(u,id,sum))         # Number of at-risk observations per county *for all 5 years*
  # quantile(y)
   
   tmp<-table(y)/N*100  # convert to %s (divide by N multiply by 100)
   barplot(tmp, ylab="Percent",xlab="Count",col="lightblue")
   
 ##########   
 # Priors #
 ##########
   alpha0<-beta0<-rep(0,p)
   T0a<-diag(.01,p)
   T0b<-diag(.01,p)         # Uniform or Gamma(0.01,0.01) prior for r depending on MH or Gibbs
   s<-0.0003                # Proposal variance  -- NOTE: may need to lower this as n_i increases

 #########   
 # Inits #
 #########
   beta<-alpha<-rep(0,p)
   phi_init<-rmvnorm(1,sigma=diag(.1,4*n))	  # Random effects
   phi_init<-matrix(phi_init,ncol=4,byrow=T)  # n x 3 matrix of spatial effects
   phi1<-phi_init[,1]
   phi2<-phi_init[,2]
   phi3<-phi_init[,3]
   phi4<-phi_init[,4]
   phi1<-phi1-mean(phi1)
   phi2<-phi2-mean(phi2)
   phi3<-phi3-mean(phi3)
   phi4<-phi4-mean(phi4)
   Phi1<-rep(phi1,nis)
   Phi2<-rep(phi2,nis)
   Phi3<-rep(phi3,nis)
   Phi4<-rep(phi4,nis)
   
   phimat<-cbind(phi1,phi2,phi3,phi4)
   Sigmaphi<-cov(phimat)
   r<-1
   Acc<-0  
   y1<-rep(0,N)             # At risk indicator (this is W in paper)
   y1[y>0]<-1               # If y>0, then at risk w.p. 1
   N0<-length(y[y==0])      # Number of observed 0's
   q<-rep(.5,N)             # 1-p=1/(1+exp(X%*%alpha)), used for updating y1
   
############
# Num Sims #
############
  nsim<-1500			          # Number of MCMC Iterations (Used 50500 in paper)
  thin<-1			              # Thinning interval
  burn<-500		              # Burnin
  lastit<-(nsim-burn)/thin	# Last stored value

#########
# Store #
#########
  Beta<-Alpha<-matrix(0,lastit,p)
  R<-R2<-rep(0,lastit)
  Sigphi<-matrix(0,lastit,16)
  PHI1<-PHI2<-PHI3<-PHI4<-matrix(0,lastit,n)
  
########
# MCMC #
########
tmptime<-proc.time()

for (i in 1:nsim){

# Update alpha  
  mu<-X%*%alpha+Phi1+Phi2*x
  w<-rpg(N,1,mu)
  z<-(y1-1/2)/w           
  v<-solve(crossprod(sqrt(w)*X)+T0a)  
  m<-v%*%(T0a%*%alpha0+t(sqrt(w)*X)%*%(sqrt(w)*(z-Phi1-Phi2*x)))
  alpha<-c(rmvnorm(1,m,v))

# Update phi1
  priorprec<-1/(Sigmaphi[1,1]-Sigmaphi[1,-1]%*%solve(Sigmaphi[-1,-1])%*%Sigmaphi[-1,1])*Q # Prior Prec of phi1|phi2,phi3,phi4
  priormean<-diag(n)%x%(Sigmaphi[1,-1]%*%solve(Sigmaphi[-1,-1]))%*%c(t(phimat[,-1]))      # Prior mean of phi1|phi2,phi3,phi4
  prec<-priorprec+as.spam(diag(tapply(w,id,sum),n,n))	
  m<-c(priorprec%*%priormean)+tapply(w*(z-X%*%alpha-Phi2*x),id,sum)
  phi1<-rmvnorm.canonical(1, m, prec)[1,]

# Center
  phi1<-phi1-mean(phi1)
  Phi1<-rep(phi1,nis)
  
# Update phi2
  priorprec<-1/(Sigmaphi[2,2]-Sigmaphi[2,-2]%*%solve(Sigmaphi[-2,-2])%*%Sigmaphi[-2,2])*Q # Prior Prec of phi2|phi1,phi3,phi4
  priormean<-diag(n)%x%(Sigmaphi[2,-2]%*%solve(Sigmaphi[-2,-2]))%*%c(t(phimat[,-2]))      # Prior mean of phi2|phi1,phi3,phi4
  prec<-priorprec+as.spam(diag(tapply(w*x^2,id,sum),n,n))	
  m<-c(priorprec%*%priormean)+tapply(x*w*(z-X%*%alpha-Phi1),id,sum)
  phi2<-rmvnorm.canonical(1, m, prec)[1,]

# Center
  phi2<-phi2-mean(phi2)
  Phi2<-rep(phi2,nis)
  
# Update at-risk indicator y1 (W in paper)
  eta1<-X%*%alpha+Phi1+Phi2*x
  eta2<-X%*%beta+Phi3+Phi4*x              # Use all n observations
  pi<-pmax(0.01,pmin(0.99,exp(eta1)/(1+exp(eta1))))  # at-risk probability
  q<-1/(1+exp(eta2))                      # Pr(y=0|y1=1)
  theta<-pi*(q^r)/(pi*(q^r)+1-pi)         # Conditional prob that y1=1 given y=0 -- i.e. Pr(chance zero|observed zero)
  y1[y==0]<-rbinom(N0,1,theta[y==0])      # If y=0, then draw a "chance zero" w.p. theta, otherwise y1=1
  N1<-sum(y1)
  nis1<-tapply(y1,id,sum)
  
# Update r
  rnew<-rtnorm(1,r,sqrt(s),lower=0)       # Treat r as continuous
  ratio<-sum(dnbinom(y[y1==1],rnew,q[y1==1],log=T))-sum(dnbinom(y[y1==1],r,q[y1==1],log=T))+   
    dtnorm(r,rnew,sqrt(s),0,log=T) - dtnorm(rnew,r,sqrt(s),0,log=T)   # Uniform Prior for R 
                                                                      # Proposal not symmetric 
  if (log(runif(1))<ratio) {
    r<-rnew
    Acc<-Acc+1
  }
  
# Update r2 using Gibbs as in Dadaneh et al and Zhou and Carin
# Update latent counts, l        
  l<-rep(0,N1)
  ytmp<-y[y1==1]
  for(j in 1:N1) l[j]<-sum(rbinom(ytmp[j],1,round(r/(r+1:ytmp[j]-1),6))) # Could try to avoid loop; rounding avoids numerical stability
  
# Update r from conjugate gamma distribution given l and psi
  psi<-exp(eta2[y1==1])/(1+exp(eta2[y1==1]))
  r2<-rgamma(1,0.01+sum(l),0.01-sum(log(1-psi)))

# Update beta
  eta<-X[y1==1,]%*%beta+Phi3[y1==1]+Phi4[y1==1]*x[y1==1]
  w<-rpg(N1,y[y1==1]+r,eta)                               # Polya weights
  z<-(y[y1==1]-r)/(2*w)                                      
  v<-solve(crossprod(X[y1==1,]*sqrt(w))+T0b)               
  m<-v%*%(T0b%*%beta0+t(sqrt(w)*X[y1==1,])%*%(sqrt(w)*(z-Phi3[y1==1]-Phi4[y1==1]*x[y1==1])))  
  beta<-c(rmvnorm(1,m,v))

# Update phi3
  n1<-length(nis1[nis1>0])
  
  priorprec<-1/(Sigmaphi[3,3]-Sigmaphi[3,-3]%*%solve(Sigmaphi[-3,-3])%*%Sigmaphi[-3,3])*Q # Prior Prec of phi3|phi1,phi2,phi4
  priormean<-diag(n)%x%(Sigmaphi[3,-3]%*%solve(Sigmaphi[-3,-3]))%*%c(t(phimat[,-3]))      # Prior mean of phi3|phi1,phi2,phi4
  prec<-priorprec+as.spam(diag(tapply(w,id[y1==1],sum),n,n))
  tmp<-rep(0,n)                       # Account empty blocks
  tmp[nis1>0]<-tapply(w*(z-X[y1==1,]%*%beta-Phi4[y1==1]*x[y1==1]),id[y1==1],sum)
  m<-c(priorprec%*%priormean)+tmp
  phi3<-rmvnorm.canonical(1, m, prec)[1,]
  
# Center
  phi3<-phi3-mean(phi3)
  Phi3<-rep(phi3,nis)
  
  # Update phi4
  priorprec<-1/(Sigmaphi[4,4]-Sigmaphi[4,-4]%*%solve(Sigmaphi[-4,-4])%*%Sigmaphi[-4,4])*Q # Prior Prec of phi4|phi1,phi2,phi3
  priormean<-diag(n)%x%(Sigmaphi[4,-4]%*%solve(Sigmaphi[-4,-4]))%*%c(t(phimat[,-4]))      # Prior mean of phi4|phi1,phi2,phi3
  
  prec<-priorprec+as.spam(diag(tapply(w*x[y1==1]^2,id[y1==1],sum),n,n))
  tmp<-rep(0,n)                       # Account for empty counties
  tmp[nis1>0]<-tapply(x[y1==1]*w*(z-X[y1==1,]%*%beta-Phi3[y1==1]),id[y1==1],sum)
  m<-c(priorprec%*%priormean)+tmp
  phi4<-rmvnorm.canonical(1, m, prec)[1,]
  
# Center
  phi4<-phi4-mean(phi4)
  Phi4<-rep(phi4,nis)
  
# Update Sigma.phi
  phimat<-cbind(phi1,phi2,phi3,phi4)
  Sigmaphi<-riwish(4+n-1,diag(4)+t(phimat)%*%Q%*%phimat)

# Store
  if (i> burn & i%%thin==0) {
    j<-(i-burn)/thin
    Alpha[j,]<-alpha
    Beta[j,]<-beta
    R[j]<-r		
    R2[j]<-r2	
    Sigphi[j,]<-c(Sigmaphi)
    PHI1[j,]<-phi1
    PHI2[j,]<-phi2
    PHI3[j,]<-phi3
    PHI4[j,]<-phi4
  }
  
  if (i%%10==0) print(i)
  
}

tot.time<-proc.time()-tmptime  # Takes approx 6.6 hours to run without R2

###########
# Results #
###########
  malpha<-colMeans(Alpha)
  salpha<-apply(Alpha,2,sd)
  qalpha<-apply(Alpha,2,quantile,prob=c(0.025,0.975))
  mbeta<-colMeans(Beta)
  sbeta<-apply(Beta,2,sd)
  qbeta<-apply(Beta,2,quantile,prob=c(0.025,0.975))
  mr<-mean(R)
  qr<-quantile(R,prob=c(0.025,0.975))
  #mr2<-mean(R2)
  #qr2<-quantile(R2,prob=c(0.025,0.975))
  msigphi<-colMeans(Sigphi)
  qsigphi<-apply(Sigphi,2,quantile,prob=c(0.025,0.975))
  mphi1<-colMeans(PHI1)
  mphi2<-colMeans(PHI2)
  mphi3<-colMeans(PHI3)
  mphi4<-colMeans(PHI4)
  mphi<-cbind(mphi1,mphi2,mphi3,mphi4)
  
# Print Results
  malpha
  mbeta
  mr
  msigphi

###############
# Trace plots #
###############
    par(mfrow=c(3,2))
  
    plot(1:lastit,Alpha[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[10]))
    abline(h=malpha[1],col="blue4")
  
    plot(1:lastit,Beta[,2],type="l",col="lightgreen",xlab="Iteration",ylab=expression(beta[21]))
    abline(h=mbeta[2],col="blue4")
    
    plot(1:lastit,R,type="l",col="lightgreen",xlab="Iteration",ylab=expression(r))
    abline(h=mr,col="blue4")
    
    plot(1:lastit,Sigphi[,1],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[11]))
    abline(h=mean(Sigphi[,1]),col="blue4")
    
    plot(1:lastit,Sigphi[,4],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[14]))
    abline(h=mean(Sigphi[,4]),col="blue4")
    
    plot(1:lastit,Sigphi[,16],type="l",col="lightgreen",xlab="Iteration",ylab=expression(Gamma[44]))
    abline(h=mean(Sigphi[,16]),col="blue4")



